@@ -1,11 +1,10 @@
 # Net::Config.pm
 #
-# Versions up to 1.11 Copyright (c) 2000 Graham Barr <gbarr@pobox.com>.
-# All rights reserved.
-# Changes in Version 1.11_01 onwards Copyright (C) 2013-2014 Steve Hay.  All
-# rights reserved.
-# This program is free software; you can redistribute it and/or
-# modify it under the same terms as Perl itself.
+# Copyright (C) 2000 Graham Barr.  All rights reserved.
+# Copyright (C) 2013-2014, 2016, 2020 Steve Hay.  All rights reserved.
+# This module is free software; you can redistribute it and/or modify it under
+# the same terms as Perl itself, i.e. under the terms of either the GNU General
+# Public License or the Artistic License, as specified in the F<LICENCE> file.
 
 package Net::Config;
 
@@ -19,11 +18,16 @@ use Socket qw(inet_aton inet_ntoa);
 
 our @EXPORT  = qw(%NetConfig);
 our @ISA     = qw(Net::LocalCfg Exporter);
-our $VERSION = "3.03";
+our $VERSION = "3.14";
 
 our($CONFIGURE, $LIBNET_CFG);
 
-eval { local $SIG{__DIE__}; require Net::LocalCfg };
+eval {
+  local @INC = @INC;
+  pop @INC if $INC[-1] eq '.';
+  local $SIG{__DIE__};
+  require Net::LocalCfg;
+};
 
 our %NetConfig = (
   nntp_hosts      => [],
@@ -133,7 +137,7 @@ __END__
 
 Net::Config - Local configuration data for libnet
 
-=head1 SYNOPSYS
+=head1 SYNOPSIS
 
     use Net::Config qw(%NetConfig);
 
@@ -155,7 +159,7 @@ For example
     }
     __END__
 
-=head1 METHODS
+=head2 Class Methods
 
 C<Net::Config> defines the following methods. They are methods as they are
 invoked as class methods. This is because C<Net::Config> inherits from
@@ -163,7 +167,7 @@ C<Net::LocalCfg> so you can override these methods if you want.
 
 =over 4
 
-=item requires_firewall ( HOST )
+=item C<requires_firewall($host)>
 
 Attempts to determine if a given host is outside your firewall. Possible
 return values are.
@@ -177,7 +181,7 @@ the configuration data.
 
 =back
 
-=head1 NetConfig VALUES
+=head2 NetConfig Values
 
 =over 4
 
@@ -319,4 +323,59 @@ If true then C<Configure> will check each hostname given that it exists
 
 =back
 
+=head1 EXPORTS
+
+The following symbols are, or can be, exported by this module:
+
+=over 4
+
+=item Default Exports
+
+C<%NetConfig>.
+
+=item Optional Exports
+
+I<None>.
+
+=item Export Tags
+
+I<None>.
+
+=back
+
+=head1 KNOWN BUGS
+
+I<None>.
+
+=head1 AUTHOR
+
+Graham Barr E<lt>L<gbarr@pobox.com|mailto:gbarr@pobox.com>E<gt>.
+
+Steve Hay E<lt>L<shay@cpan.org|mailto:shay@cpan.org>E<gt> is now maintaining
+libnet as of version 1.22_02.
+
+=head1 COPYRIGHT
+
+Copyright (C) 2000 Graham Barr.  All rights reserved.
+
+Copyright (C) 2013-2014, 2016, 2020 Steve Hay.  All rights reserved.
+
+=head1 LICENCE
+
+This module is free software; you can redistribute it and/or modify it under the
+same terms as Perl itself, i.e. under the terms of either the GNU General Public
+License or the Artistic License, as specified in the F<LICENCE> file.
+
+=head1 VERSION
+
+Version 3.14
+
+=head1 DATE
+
+22 May 2022
+
+=head1 HISTORY
+
+See the F<Changes> file.
+
 =cut
