@@ -1,11 +1,10 @@
 # Net::NNTP.pm
 #
-# Versions up to 2.24_1 Copyright (c) 1995-1997 Graham Barr <gbarr@pobox.com>.
-# All rights reserved.
-# Changes in Version 2.25 onwards Copyright (C) 2013-2014 Steve Hay.  All rights
-# reserved.
-# This program is free software; you can redistribute it and/or
-# modify it under the same terms as Perl itself.
+# Copyright (C) 1995-1997 Graham Barr.  All rights reserved.
+# Copyright (C) 2013-2016, 2020 Steve Hay.  All rights reserved.
+# This module is free software; you can redistribute it and/or modify it under
+# the same terms as Perl itself, i.e. under the terms of either the GNU General
+# Public License or the Artistic License, as specified in the F<LICENCE> file.
 
 package Net::NNTP;
 
@@ -20,7 +19,7 @@ use Net::Cmd;
 use Net::Config;
 use Time::Local;
 
-our $VERSION = "3.03";
+our $VERSION = "3.14";
 
 # Code for detecting if we can use SSL
 my $ssl_class = eval {
@@ -34,16 +33,19 @@ my $nossl_warn = !$ssl_class &&
   'To use SSL please install IO::Socket::SSL with version>=2.007';
 
 # Code for detecting if we can use IPv6
+my $family_key = 'Domain';
 my $inet6_class = eval {
   require IO::Socket::IP;
   no warnings 'numeric';
-  IO::Socket::IP->VERSION(0.20);
+  IO::Socket::IP->VERSION(0.25) || die;
+  $family_key = 'Family';
 } && 'IO::Socket::IP' || eval {
   require IO::Socket::INET6;
   no warnings 'numeric';
   IO::Socket::INET6->VERSION(2.62);
 } && 'IO::Socket::INET6';
 
+
 sub can_ssl   { $ssl_class };
 sub can_inet6 { $inet6_class };
 
@@ -80,9 +82,10 @@ sub new {
     $connect{$_} = $arg{$_} for(grep { m{^SSL_} } keys %arg);
   }
 
-  foreach my $o (qw(LocalAddr Timeout)) {
+  foreach my $o (qw(LocalAddr LocalPort Timeout)) {
     $connect{$o} = $arg{$o} if exists $arg{$o};
   }
+  $connect{$family_key} = $arg{Domain} || $arg{Family};
   $connect{Timeout} = 120 unless defined $connect{Timeout};
   $connect{PeerPort} = $arg{Port} || 'nntp(119)';
   foreach my $h (@{$hosts}) {
@@ -93,7 +96,6 @@ sub new {
     if ($arg{SSL}) {
       Net::NNTP::_SSL->start_SSL($obj,%arg) or next;
     }
-    last:
   }
 
   return
@@ -173,7 +175,7 @@ sub starttls {
 
 
 sub article {
-  @_ >= 1 && @_ <= 3 or croak 'usage: $nntp->article( [ MSGID ], [ FH ] )';
+  @_ >= 1 && @_ <= 3 or croak 'usage: $nntp->article([{$msgid|$msgnum}[, $fh]])';
   my $nntp = shift;
   my @fh;
 
@@ -186,7 +188,7 @@ sub article {
 
 
 sub articlefh {
-  @_ >= 1 && @_ <= 2 or croak 'usage: $nntp->articlefh( [ MSGID ] )';
+  @_ >= 1 && @_ <= 2 or croak 'usage: $nntp->articlefh([{$msgid|$msgnum}])';
   my $nntp = shift;
 
   return unless $nntp->_ARTICLE(@_);
@@ -195,7 +197,7 @@ sub articlefh {
 
 
 sub authinfo {
-  @_ == 3 or croak 'usage: $nntp->authinfo( USER, PASS )';
+  @_ == 3 or croak 'usage: $nntp->authinfo($user, $pass)';
   my ($nntp, $user, $pass) = @_;
 
   $nntp->_AUTHINFO("USER",      $user) == CMD_MORE
@@ -204,7 +206,7 @@ sub authinfo {
 
 
 sub authinfo_simple {
-  @_ == 3 or croak 'usage: $nntp->authinfo( USER, PASS )';
+  @_ == 3 or croak 'usage: $nntp->authinfo_simple($user, $pass)';
   my ($nntp, $user, $pass) = @_;
 
   $nntp->_AUTHINFO('SIMPLE') == CMD_MORE
@@ -213,7 +215,7 @@ sub authinfo_simple {
 
 
 sub body {
-  @_ >= 1 && @_ <= 3 or croak 'usage: $nntp->body( [ MSGID ], [ FH ] )';
+  @_ >= 1 && @_ <= 3 or croak 'usage: $nntp->body([{$msgid|$msgnum}[, $fh]])';
   my $nntp = shift;
   my @fh;
 
@@ -226,7 +228,7 @@ sub body {
 
 
 sub bodyfh {
-  @_ >= 1 && @_ <= 2 or croak 'usage: $nntp->bodyfh( [ MSGID ] )';
+  @_ >= 1 && @_ <= 2 or croak 'usage: $nntp->bodyfh([{$msgid|$msgnum}])';
   my $nntp = shift;
   return unless $nntp->_BODY(@_);
   return $nntp->tied_fh;
@@ -234,7 +236,7 @@ sub bodyfh {
 
 
 sub head {
-  @_ >= 1 && @_ <= 3 or croak 'usage: $nntp->head( [ MSGID ], [ FH ] )';
+  @_ >= 1 && @_ <= 3 or croak 'usage: $nntp->head([{$msgid|$msgnum}[, $fh]])';
   my $nntp = shift;
   my @fh;
 
@@ -247,7 +249,7 @@ sub head {
 
 
 sub headfh {
-  @_ >= 1 && @_ <= 2 or croak 'usage: $nntp->headfh( [ MSGID ] )';
+  @_ >= 1 && @_ <= 2 or croak 'usage: $nntp->headfh([{$msgid|$msgnum}])';
   my $nntp = shift;
   return unless $nntp->_HEAD(@_);
   return $nntp->tied_fh;
@@ -255,7 +257,7 @@ sub headfh {
 
 
 sub nntpstat {
-  @_ == 1 || @_ == 2 or croak 'usage: $nntp->nntpstat( [ MSGID ] )';
+  @_ == 1 || @_ == 2 or croak 'usage: $nntp->nntpstat([{$msgid|$msgnum}])';
   my $nntp = shift;
 
   $nntp->_STAT(@_) && $nntp->message =~ /(<[^>]+>)/o
@@ -265,7 +267,7 @@ sub nntpstat {
 
 
 sub group {
-  @_ == 1 || @_ == 2 or croak 'usage: $nntp->group( [ GROUP ] )';
+  @_ == 1 || @_ == 2 or croak 'usage: $nntp->group([$group])';
   my $nntp = shift;
   my $grp  = ${*$nntp}{'net_nntp_group'};
 
@@ -305,11 +307,11 @@ sub help {
 
 
 sub ihave {
-  @_ >= 2 or croak 'usage: $nntp->ihave( MESSAGE-ID [, MESSAGE ])';
-  my $nntp = shift;
-  my $mid  = shift;
+  @_ >= 2 or croak 'usage: $nntp->ihave($msgid[, $message])';
+  my $nntp  = shift;
+  my $msgid = shift;
 
-  $nntp->_IHAVE($mid) && $nntp->datasend(@_)
+  $nntp->_IHAVE($msgid) && $nntp->datasend(@_)
     ? @_ == 0 || $nntp->dataend
     : undef;
 }
@@ -336,15 +338,15 @@ sub list {
 
 
 sub newgroups {
-  @_ >= 2 or croak 'usage: $nntp->newgroups( SINCE [, DISTRIBUTIONS ])';
+  @_ >= 2 or croak 'usage: $nntp->newgroups($since[, $distributions])';
   my $nntp = shift;
-  my $time = _timestr(shift);
-  my $dist = shift || "";
+  my $since = _timestr(shift);
+  my $distributions = shift || "";
 
-  $dist = join(",", @{$dist})
-    if ref($dist);
+  $distributions = join(",", @{$distributions})
+    if ref($distributions);
 
-  $nntp->_NEWGROUPS($time, $dist)
+  $nntp->_NEWGROUPS($since, $distributions)
     ? $nntp->_grouplist
     : undef;
 }
@@ -352,20 +354,20 @@ sub newgroups {
 
 sub newnews {
   @_ >= 2 && @_ <= 4
-    or croak 'usage: $nntp->newnews( SINCE [, GROUPS [, DISTRIBUTIONS ]])';
+    or croak 'usage: $nntp->newnews($since[, $groups[, $distributions]])';
   my $nntp = shift;
-  my $time = _timestr(shift);
-  my $grp  = @_ ? shift: $nntp->group;
-  my $dist = shift || "";
+  my $since = _timestr(shift);
+  my $groups = @_ ? shift : $nntp->group;
+  my $distributions = shift || "";
 
-  $grp ||= "*";
-  $grp = join(",", @{$grp})
-    if ref($grp);
+  $groups ||= "*";
+  $groups = join(",", @{$groups})
+    if ref($groups);
 
-  $dist = join(",", @{$dist})
-    if ref($dist);
+  $distributions = join(",", @{$distributions})
+    if ref($distributions);
 
-  $nntp->_NEWNEWS($grp, $time, $dist)
+  $nntp->_NEWNEWS($groups, $since, $distributions)
     ? $nntp->_articlelist
     : undef;
 }
@@ -382,7 +384,7 @@ sub next {
 
 
 sub post {
-  @_ >= 1 or croak 'usage: $nntp->post( [ MESSAGE ] )';
+  @_ >= 1 or croak 'usage: $nntp->post([$message])';
   my $nntp = shift;
 
   $nntp->_POST() && $nntp->datasend(@_)
@@ -420,7 +422,7 @@ sub slave {
 
 
 sub active {
-  @_ == 1 || @_ == 2 or croak 'usage: $nntp->active( [ PATTERN ] )';
+  @_ == 1 || @_ == 2 or croak 'usage: $nntp->active([$pattern])';
   my $nntp = shift;
 
   $nntp->_LIST('ACTIVE', @_)
@@ -450,7 +452,7 @@ sub distributions {
 
 
 sub distribution_patterns {
-  @_ == 1 or croak 'usage: $nntp->distributions()';
+  @_ == 1 or croak 'usage: $nntp->distribution_patterns()';
   my $nntp = shift;
 
   my $arr;
@@ -465,7 +467,7 @@ sub distribution_patterns {
 
 
 sub newsgroups {
-  @_ == 1 || @_ == 2 or croak 'usage: $nntp->newsgroups( [ PATTERN ] )';
+  @_ == 1 || @_ == 2 or croak 'usage: $nntp->newsgroups([$pattern])';
   my $nntp = shift;
 
   $nntp->_LIST('NEWSGROUPS', @_)
@@ -495,7 +497,7 @@ sub subscriptions {
 
 
 sub listgroup {
-  @_ == 1 || @_ == 2 or croak 'usage: $nntp->listgroup( [ GROUP ] )';
+  @_ == 1 || @_ == 2 or croak 'usage: $nntp->listgroup([$group])';
   my $nntp = shift;
 
   $nntp->_LISTGROUP(@_)
@@ -513,7 +515,7 @@ sub reader {
 
 
 sub xgtitle {
-  @_ == 1 || @_ == 2 or croak 'usage: $nntp->xgtitle( [ PATTERN ] )';
+  @_ == 1 || @_ == 2 or croak 'usage: $nntp->xgtitle([$pattern])';
   my $nntp = shift;
 
   $nntp->_XGTITLE(@_)
@@ -523,19 +525,19 @@ sub xgtitle {
 
 
 sub xhdr {
-  @_ >= 2 && @_ <= 4 or croak 'usage: $nntp->xhdr( HEADER, [ MESSAGE-SPEC ] )';
+  @_ >= 2 && @_ <= 4 or croak 'usage: $nntp->xhdr($header[, $message_spec])';
   my $nntp = shift;
-  my $hdr  = shift;
-  my $arg  = _msg_arg(@_);
+  my $header = shift;
+  my $arg = _msg_arg(@_);
 
-  $nntp->_XHDR($hdr, $arg)
+  $nntp->_XHDR($header, $arg)
     ? $nntp->_description
     : undef;
 }
 
 
 sub xover {
-  @_ == 2 || @_ == 3 or croak 'usage: $nntp->xover( MESSAGE-SPEC )';
+  @_ == 2 || @_ == 3 or croak 'usage: $nntp->xover($message_spec)';
   my $nntp = shift;
   my $arg  = _msg_arg(@_);
 
@@ -546,27 +548,27 @@ sub xover {
 
 
 sub xpat {
-  @_ == 4 || @_ == 5 or croak '$nntp->xpat( HEADER, PATTERN, MESSAGE-SPEC )';
+  @_ == 4 || @_ == 5 or croak 'usage: $nntp->xpat($header, $pattern, $message_spec )';
   my $nntp = shift;
-  my $hdr  = shift;
-  my $pat  = shift;
-  my $arg  = _msg_arg(@_);
+  my $header = shift;
+  my $pattern = shift;
+  my $arg = _msg_arg(@_);
 
-  $pat = join(" ", @$pat)
-    if ref($pat);
+  $pattern = join(" ", @$pattern)
+    if ref($pattern);
 
-  $nntp->_XPAT($hdr, $arg, $pat)
+  $nntp->_XPAT($header, $arg, $pattern)
     ? $nntp->_description
     : undef;
 }
 
 
 sub xpath {
-  @_ == 2 or croak 'usage: $nntp->xpath( MESSAGE-ID )';
-  my ($nntp, $mid) = @_;
+  @_ == 2 or croak 'usage: $nntp->xpath($message_id)';
+  my ($nntp, $message_id) = @_;
 
   return
-    unless $nntp->_XPATH($mid);
+    unless $nntp->_XPATH($message_id);
 
   my $m;
   ($m = $nntp->message) =~ s/^\d+\s+//o;
@@ -577,7 +579,7 @@ sub xpath {
 
 
 sub xrover {
-  @_ == 2 || @_ == 3 or croak 'usage: $nntp->xrover( MESSAGE-SPEC )';
+  @_ == 2 || @_ == 3 or croak 'usage: $nntp->xrover($message_spec)';
   my $nntp = shift;
   my $arg  = _msg_arg(@_);
 
@@ -593,7 +595,7 @@ sub date {
 
   $nntp->_DATE
     && $nntp->message =~ /(\d{4})(\d\d)(\d\d)(\d\d)(\d\d)(\d\d)/
-    ? timegm($6, $5, $4, $3, $2 - 1, $1 - 1900)
+    ? timegm($6, $5, $4, $3, $2 - 1, $1)
     : undef;
 }
 
@@ -756,9 +758,9 @@ sub DESTROY {
     my ($class,$nntp,%arg) = @_;
     delete @arg{ grep { !m{^SSL_} } keys %arg };
     ( $arg{SSL_verifycn_name} ||= $nntp->host )
-	=~s{(?<!:):[\w()]+$}{}; # strip port
+        =~s{(?<!:):[\w()]+$}{}; # strip port
     $arg{SSL_hostname} = $arg{SSL_verifycn_name}
-	if ! defined $arg{SSL_hostname};
+        if ! defined $arg{SSL_hostname} && $class->can_client_sni;
     my $ok = $class->SUPER::start_SSL($nntp,
       SSL_verifycn_scheme => 'nntp',
       %arg
@@ -798,23 +800,26 @@ Net::NNTP - NNTP Client class
 
 C<Net::NNTP> is a class implementing a simple NNTP client in Perl as described
 in RFC977 and RFC4642.
+With L<IO::Socket::SSL> installed it also provides support for implicit and
+explicit TLS encryption, i.e. NNTPS or NNTP+STARTTLS.
 
-The Net::NNTP class is a subclass of Net::Cmd and IO::Socket::INET.
+The Net::NNTP class is a subclass of Net::Cmd and (depending on avaibility) of
+IO::Socket::IP, IO::Socket::INET6 or IO::Socket::INET.
 
-=head1 CONSTRUCTOR
+=head2 Class Methods
 
 =over 4
 
-=item new ( [ HOST ] [, OPTIONS ])
+=item C<new([$host][, %options])>
 
-This is the constructor for a new Net::NNTP object. C<HOST> is the
+This is the constructor for a new Net::NNTP object. C<$host> is the
 name of the remote host to which a NNTP connection is required. If not
 given then it may be passed as the C<Host> option described below. If no host is passed
 then two environment variables are checked, first C<NNTPSERVER> then
 C<NEWSHOST>, then C<Net::Config> is checked, and if a host is not found
 then C<news> is used.
 
-C<OPTIONS> are passed in a hash like fashion, using key and value pairs.
+C<%options> are passed in a hash like fashion, using key and value pairs.
 Possible options are:
 
 B<Host> - NNTP host to connect to. It may be a single scalar, as defined for
@@ -837,19 +842,21 @@ NNTP server, a value of zero will cause all IO operations to block.
 B<Debug> - Enable the printing of debugging information to STDERR
 
 B<Reader> - If the remote server is INN then initially the connection
-will be to nnrpd, by default C<Net::NNTP> will issue a C<MODE READER> command
-so that the remote server becomes innd. If the C<Reader> option is given
+will be to innd, by default C<Net::NNTP> will issue a C<MODE READER> command
+so that the remote server becomes nnrpd. If the C<Reader> option is given
 with a value of zero, then this command will not be sent and the
-connection will be left talking to nnrpd.
+connection will be left talking to innd.
 
-B<LocalAddr> - If multiple IP addresses are present on the client host
-with a valid route to the destination, you can specify the address your
-C<Net::NNTP> connects from and this way override the operating system's
-pick.
+B<LocalAddr> and B<LocalPort> - These parameters are passed directly
+to IO::Socket to allow binding the socket to a specific local address and port.
+
+B<Domain> - This parameter is passed directly to IO::Socket and makes it
+possible to enforce IPv4 connections even if L<IO::Socket::IP> is used as super
+class. Alternatively B<Family> can be used.
 
 =back
 
-=head1 METHODS
+=head2 Object Methods
 
 Unless otherwise stated all methods return either a I<true> or I<false>
 value, with I<true> meaning that the operation was a success. When a method
@@ -862,58 +869,58 @@ documented here.
 
 =over 4
 
-=item host ()
+=item C<host()>
 
 Returns the value used by the constructor, and passed to IO::Socket::INET,
 to connect to the host.
 
-=item starttls ()
+=item C<starttls()>
 
 Upgrade existing plain connection to SSL.
 Any arguments necessary for SSL must be given in C<new> already.
 
-=item article ( [ MSGID|MSGNUM ], [FH] )
+=item C<article([{$msgid|$msgnum}[, $fh]])>
 
 Retrieve the header, a blank line, then the body (text) of the
 specified article. 
 
-If C<FH> is specified then it is expected to be a valid filehandle
+If C<$fh> is specified then it is expected to be a valid filehandle
 and the result will be printed to it, on success a true value will be
-returned. If C<FH> is not specified then the return value, on success,
+returned. If C<$fh> is not specified then the return value, on success,
 will be a reference to an array containing the article requested, each
 entry in the array will contain one line of the article.
 
 If no arguments are passed then the current article in the currently
 selected newsgroup is fetched.
 
-C<MSGNUM> is a numeric id of an article in the current newsgroup, and
-will change the current article pointer.  C<MSGID> is the message id of
+C<$msgnum> is a numeric id of an article in the current newsgroup, and
+will change the current article pointer.  C<$msgid> is the message id of
 an article as shown in that article's header.  It is anticipated that the
-client will obtain the C<MSGID> from a list provided by the C<newnews>
+client will obtain the C<$msgid> from a list provided by the C<newnews>
 command, from references contained within another article, or from the
 message-id provided in the response to some other commands.
 
 If there is an error then C<undef> will be returned.
 
-=item body ( [ MSGID|MSGNUM ], [FH] )
+=item C<body([{$msgid|$msgnum}[, [$fh]])>
 
 Like C<article> but only fetches the body of the article.
 
-=item head ( [ MSGID|MSGNUM ], [FH] )
+=item C<head([{$msgid|$msgnum}[, [$fh]])>
 
 Like C<article> but only fetches the headers for the article.
 
-=item articlefh ( [ MSGID|MSGNUM ] )
+=item C<articlefh([{$msgid|$msgnum}])>
 
-=item bodyfh ( [ MSGID|MSGNUM ] )
+=item C<bodyfh([{$msgid|$msgnum}])>
 
-=item headfh ( [ MSGID|MSGNUM ] )
+=item C<headfh([{$msgid|$msgnum}])>
 
 These are similar to article(), body() and head(), but rather than
 returning the requested data directly, they return a tied filehandle
 from which to read the article.
 
-=item nntpstat ( [ MSGID|MSGNUM ] )
+=item C<nntpstat([{$msgid|$msgnum}])>
 
 The C<nntpstat> command is similar to the C<article> command except that no
 text is returned.  When selecting by message number within a group,
@@ -926,9 +933,9 @@ selection by message-id does B<not> alter the "current article pointer".
 
 Returns the message-id of the "current article".
 
-=item group ( [ GROUP ] )
+=item C<group([$group])>
 
-Set and/or get the current group. If C<GROUP> is not given then information
+Set and/or get the current group. If C<$group> is not given then information
 is returned on the current group.
 
 In a scalar context it returns the group name.
@@ -937,43 +944,45 @@ In an array context the return value is a list containing, the number
 of articles in the group, the number of the first article, the number
 of the last article and the group name.
 
-=item help ( )
+=item C<help()>
 
 Request help text (a short summary of commands that are understood by this
 implementation) from the server. Returns the text or undef upon failure.
 
-=item ihave ( MSGID [, MESSAGE ])
+=item C<ihave($msgid[, $message])>
 
 The C<ihave> command informs the server that the client has an article
-whose id is C<MSGID>.  If the server desires a copy of that
-article, and C<MESSAGE> has been given the it will be sent.
+whose id is C<$msgid>.  If the server desires a copy of that
+article and C<$message> has been given then it will be sent.
 
-Returns I<true> if the server desires the article and C<MESSAGE> was
-successfully sent,if specified.
+Returns I<true> if the server desires the article and C<$message> was
+successfully sent, if specified.
 
-If C<MESSAGE> is not specified then the message must be sent using the
+If C<$message> is not specified then the message must be sent using the
 C<datasend> and C<dataend> methods from L<Net::Cmd>
 
-C<MESSAGE> can be either an array of lines or a reference to an array.
+C<$message> can be either an array of lines or a reference to an array
+and must be encoded by the caller to octets of whatever encoding is required,
+e.g. by using the Encode module's C<encode()> function.
 
-=item last ()
+=item C<last()>
 
 Set the "current article pointer" to the previous article in the current
 newsgroup.
 
 Returns the message-id of the article.
 
-=item date ()
+=item C<date()>
 
 Returns the date on the remote server. This date will be in a UNIX time
 format (seconds since 1970)
 
-=item postok ()
+=item C<postok()>
 
 C<postok> will return I<true> if the servers initial response indicated
 that it will allow posting.
 
-=item authinfo ( USER, PASS )
+=item C<authinfo($user, $pass)>
 
 Authenticates to the server (using the original AUTHINFO USER / AUTHINFO PASS
 form, defined in RFC2980) using the supplied username and password.  Please
@@ -981,59 +990,61 @@ note that the password is sent in clear text to the server.  This command
 should not be used with valuable passwords unless the connection to the server
 is somehow protected.
 
-=item authinfo_simple ( USER, PASS )
+=item C<authinfo_simple($user, $pass)>
 
 Authenticates to the server (using the proposed NNTP V2 AUTHINFO SIMPLE form,
 defined and deprecated in RFC2980) using the supplied username and password.
 As with L</authinfo> the password is sent in clear text.
 
-=item list ()
+=item C<list()>
 
 Obtain information about all the active newsgroups. The results is a reference
 to a hash where the key is a group name and each value is a reference to an
 array. The elements in this array are:- the last article number in the group,
 the first article number in the group and any information flags about the group.
 
-=item newgroups ( SINCE [, DISTRIBUTIONS ])
+=item C<newgroups($since[, $distributions])>
 
-C<SINCE> is a time value and C<DISTRIBUTIONS> is either a distribution
+C<$since> is a time value and C<$distributions> is either a distribution
 pattern or a reference to a list of distribution patterns.
 The result is the same as C<list>, but the
-groups return will be limited to those created after C<SINCE> and, if
-specified, in one of the distribution areas in C<DISTRIBUTIONS>. 
+groups return will be limited to those created after C<$since> and, if
+specified, in one of the distribution areas in C<$distributions>. 
 
-=item newnews ( SINCE [, GROUPS [, DISTRIBUTIONS ]])
+=item C<newnews($since[, $groups[, $distributions]])>
 
-C<SINCE> is a time value. C<GROUPS> is either a group pattern or a reference
-to a list of group patterns. C<DISTRIBUTIONS> is either a distribution
+C<$since> is a time value. C<$groups> is either a group pattern or a reference
+to a list of group patterns. C<$distributions> is either a distribution
 pattern or a reference to a list of distribution patterns.
 
 Returns a reference to a list which contains the message-ids of all news posted
-after C<SINCE>, that are in a groups which matched C<GROUPS> and a
-distribution which matches C<DISTRIBUTIONS>.
+after C<$since>, that are in a groups which matched C<$groups> and a
+distribution which matches C<$distributions>.
 
-=item next ()
+=item C<next()>
 
 Set the "current article pointer" to the next article in the current
 newsgroup.
 
 Returns the message-id of the article.
 
-=item post ( [ MESSAGE ] )
+=item C<post([$message])>
 
-Post a new article to the news server. If C<MESSAGE> is specified and posting
+Post a new article to the news server. If C<$message> is specified and posting
 is allowed then the message will be sent.
 
-If C<MESSAGE> is not specified then the message must be sent using the
+If C<$message> is not specified then the message must be sent using the
 C<datasend> and C<dataend> methods from L<Net::Cmd>
 
-C<MESSAGE> can be either an array of lines or a reference to an array.
+C<$message> can be either an array of lines or a reference to an array
+and must be encoded by the caller to octets of whatever encoding is required,
+e.g. by using the Encode module's C<encode()> function.
 
-The message, either sent via C<datasend> or as the C<MESSAGE>
+The message, either sent via C<datasend> or as the C<$message>
 parameter, must be in the format as described by RFC822 and must
 contain From:, Newsgroups: and Subject: headers.
 
-=item postfh ()
+=item C<postfh()>
 
 Post a new article to the news server using a tied filehandle.  If
 posting is allowed, this method will return a tied filehandle that you
@@ -1042,85 +1053,85 @@ explicitly close() the filehandle when you are finished posting the
 article, and the return value from the close() call will indicate
 whether the message was successfully posted.
 
-=item slave ()
+=item C<slave()>
 
 Tell the remote server that I am not a user client, but probably another
 news server.
 
-=item quit ()
+=item C<quit()>
 
 Quit the remote server and close the socket connection.
 
-=item can_inet6 ()
+=item C<can_inet6()>
 
 Returns whether we can use IPv6.
 
-=item can_ssl ()
+=item C<can_ssl()>
 
 Returns whether we can use SSL.
 
 =back
 
-=head2 Extension methods
+=head2 Extension Methods
 
 These methods use commands that are not part of the RFC977 documentation. Some
 servers may not support all of them.
 
 =over 4
 
-=item newsgroups ( [ PATTERN ] )
+=item C<newsgroups([$pattern])>
 
 Returns a reference to a hash where the keys are all the group names which
-match C<PATTERN>, or all of the groups if no pattern is specified, and
+match C<$pattern>, or all of the groups if no pattern is specified, and
 each value contains the description text for the group.
 
-=item distributions ()
+=item C<distributions()>
 
 Returns a reference to a hash where the keys are all the possible
 distribution names and the values are the distribution descriptions.
 
-=item distribution_patterns ()
+=item C<distribution_patterns()>
 
 Returns a reference to an array where each element, itself an array
 reference, consists of the three fields of a line of the distrib.pats list
 maintained by some NNTP servers, namely: a weight, a wildmat and a value
 which the client may use to construct a Distribution header.
 
-=item subscriptions ()
+=item C<subscriptions()>
 
 Returns a reference to a list which contains a list of groups which
 are recommended for a new user to subscribe to.
 
-=item overview_fmt ()
+=item C<overview_fmt()>
 
 Returns a reference to an array which contain the names of the fields returned
 by C<xover>.
 
-=item active_times ()
+=item C<active_times()>
 
 Returns a reference to a hash where the keys are the group names and each
 value is a reference to an array containing the time the groups was created
 and an identifier, possibly an Email address, of the creator.
 
-=item active ( [ PATTERN ] )
+=item C<active([$pattern])>
 
 Similar to C<list> but only active groups that match the pattern are returned.
-C<PATTERN> can be a group pattern.
+C<$pattern> can be a group pattern.
 
-=item xgtitle ( PATTERN )
+=item C<xgtitle($pattern)>
 
 Returns a reference to a hash where the keys are all the group names which
-match C<PATTERN> and each value is the description text for the group.
+match C<$pattern> and each value is the description text for the group.
 
-=item xhdr ( HEADER, MESSAGE-SPEC )
+=item C<xhdr($header, $message_spec)>
 
-Obtain the header field C<HEADER> for all the messages specified. 
+Obtain the header field C<$header> for all the messages specified. 
 
 The return value will be a reference
 to a hash where the keys are the message numbers and each value contains
 the text of the requested header for that message.
 
-=item xover ( MESSAGE-SPEC )
+=item C<xover($message_spec)>
 
 The return value will be a reference
 to a hash where the keys are the message numbers and each value contains
@@ -1129,17 +1140,17 @@ message.
 
 The names of the fields can be obtained by calling C<overview_fmt>.
 
-=item xpath ( MESSAGE-ID )
+=item C<xpath($message_id)>
 
 Returns the path name to the file on the server which contains the specified
 message.
 
-=item xpat ( HEADER, PATTERN, MESSAGE-SPEC)
+=item C<xpat($header, $pattern, $message_spec)>
 
 The result is the same as C<xhdr> except the is will be restricted to
-headers where the text of the header matches C<PATTERN>
+headers where the text of the header matches C<$pattern>
 
-=item xrover ()
+=item C<xrover($message_spec)>
 
 The XROVER command returns reference information for the article(s)
 specified.
@@ -1147,12 +1158,12 @@ specified.
 Returns a reference to a HASH where the keys are the message numbers and the
 values are the References: lines from the articles
 
-=item listgroup ( [ GROUP ] )
+=item C<listgroup([$group])>
 
-Returns a reference to a list of all the active messages in C<GROUP>, or
-the current group if C<GROUP> is not specified.
+Returns a reference to a list of all the active messages in C<$group>, or
+the current group if C<$group> is not specified.
 
-=item reader ()
+=item C<reader()>
 
 Tell the server that you are a reader and not another server.
 
@@ -1167,7 +1178,7 @@ the response is harmless.
 
 =back
 
-=head1 UNSUPPORTED
+=head2 Unsupported
 
 The following NNTP command are unsupported by the package, and there are
 no plans to do so.
@@ -1177,16 +1188,16 @@ no plans to do so.
     XSEARCH
     XINDEX
 
-=head1 DEFINITIONS
+=head2 Definitions
 
 =over 4
 
-=item MESSAGE-SPEC
+=item $message_spec
 
-C<MESSAGE-SPEC> is either a single message-id, a single message number, or
+C<$message_spec> is either a single message-id, a single message number, or
 a reference to a list of two message numbers.
 
-If C<MESSAGE-SPEC> is a reference to a list of two message numbers and the
+If C<$message_spec> is a reference to a list of two message numbers and the
 second number in a range is less than or equal to the first then the range
 represents all messages in the group after the first message number.
 
@@ -1194,7 +1205,7 @@ B<NOTE> For compatibility reasons only with earlier versions of Net::NNTP
 a message spec can be passed as a list of two numbers, this is deprecated
 and a reference to the list should now be passed
 
-=item PATTERN
+=item $pattern
 
 The C<NNTP> protocol uses the C<WILDMAT> format for patterns.
 The WILDMAT format was first developed by Rich Salz based on
@@ -1263,25 +1274,48 @@ with a and ends with d.
 
 =back
 
+=head1 EXPORTS
+
+I<None>.
+
+=head1 KNOWN BUGS
+
+See L<https://rt.cpan.org/Dist/Display.html?Status=Active&Queue=libnet>.
+
 =head1 SEE ALSO
 
 L<Net::Cmd>,
-L<IO::Socket::SSL>
+L<IO::Socket::SSL>.
 
 =head1 AUTHOR
 
-Graham Barr E<lt>F<gbarr@pobox.com>E<gt>
+Graham Barr E<lt>L<gbarr@pobox.com|mailto:gbarr@pobox.com>E<gt>.
 
-Steve Hay E<lt>F<shay@cpan.org>E<gt> is now maintaining libnet as of version
-1.22_02
+Steve Hay E<lt>L<shay@cpan.org|mailto:shay@cpan.org>E<gt> is now maintaining
+libnet as of version 1.22_02.
 
 =head1 COPYRIGHT
 
-Versions up to 2.24_1 Copyright (c) 1995-1997 Graham Barr. All rights reserved.
-Changes in Version 2.25 onwards Copyright (C) 2013-2014 Steve Hay.  All rights
-reserved.
+Copyright (C) 1995-1997 Graham Barr.  All rights reserved.
 
-This program is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself.
+Copyright (C) 2013-2016, 2020 Steve Hay.  All rights reserved.
+
+=head1 LICENCE
+
+This module is free software; you can redistribute it and/or modify it under the
+same terms as Perl itself, i.e. under the terms of either the GNU General Public
+License or the Artistic License, as specified in the F<LICENCE> file.
+
+=head1 VERSION
+
+Version 3.14
+
+=head1 DATE
+
+22 May 2022
+
+=head1 HISTORY
+
+See the F<Changes> file.
 
 =cut
