@@ -1,11 +1,10 @@
 # Net::Netrc.pm
 #
-# Versions up to 2.13 Copyright (c) 1995-1998 Graham Barr <gbarr@pobox.com>.
-# All rights reserved.
-# Changes in Version 2.13_01 onwards Copyright (C) 2013-2014 Steve Hay.  All
-# rights reserved.
-# This program is free software; you can redistribute it and/or
-# modify it under the same terms as Perl itself.
+# Copyright (C) 1995-1998 Graham Barr.  All rights reserved.
+# Copyright (C) 2013-2014, 2020 Steve Hay.  All rights reserved.
+# This module is free software; you can redistribute it and/or modify it under
+# the same terms as Perl itself, i.e. under the terms of either the GNU General
+# Public License or the Artistic License, as specified in the F<LICENCE> file.
 
 package Net::Netrc;
 
@@ -17,7 +16,7 @@ use warnings;
 use Carp;
 use FileHandle;
 
-our $VERSION = "3.03";
+our $VERSION = "3.14";
 
 our $TESTING;
 
@@ -225,7 +224,7 @@ second the ownership permissions should be such that only the owner has
 read and write access. If these conditions are not met then a warning is
 output and the .netrc file is not read.
 
-=head1 THE .netrc FILE
+=head2 The F<.netrc> File
 
 The .netrc file contains login and initialization information used by the
 auto-login process.  It resides in the user's home directory.  The following
@@ -277,7 +276,7 @@ with I<ftp>.
 
 =back
 
-=head1 CONSTRUCTOR
+=head2 Class Methods
 
 The constructor for a C<Net::Netrc> object is not called new as it does not
 really create a new object. But instead is called C<lookup> as this is
@@ -285,11 +284,11 @@ essentially what it does.
 
 =over 4
 
-=item lookup ( MACHINE [, LOGIN ])
+=item C<lookup($machine[, $login])>
 
-Lookup and return a reference to the entry for C<MACHINE>. If C<LOGIN> is given
-then the entry returned will have the given login. If C<LOGIN> is not given then
-the first entry in the .netrc file for C<MACHINE> will be returned.
+Lookup and return a reference to the entry for C<$machine>. If C<$login> is given
+then the entry returned will have the given login. If C<$login> is not given then
+the first entry in the .netrc file for C<$machine> will be returned.
 
 If a matching entry cannot be found, and a default entry exists, then a
 reference to the default entry is returned.
@@ -299,47 +298,69 @@ no .netrc file is found, then C<undef> is returned.
 
 =back
 
-=head1 METHODS
+=head2 Object Methods
 
 =over 4
 
-=item login ()
+=item C<login()>
 
 Return the login id for the netrc entry
 
-=item password ()
+=item C<password()>
 
 Return the password for the netrc entry
 
-=item account ()
+=item C<account()>
 
 Return the account information for the netrc entry
 
-=item lpa ()
+=item C<lpa()>
 
 Return a list of login, password and account information for the netrc entry
 
 =back
 
-=head1 AUTHOR
+=head1 EXPORTS
 
-Graham Barr E<lt>F<gbarr@pobox.com>E<gt>
+I<None>.
 
-Steve Hay E<lt>F<shay@cpan.org>E<gt> is now maintaining libnet as of version
-1.22_02
+=head1 KNOWN BUGS
+
+See L<https://rt.cpan.org/Dist/Display.html?Status=Active&Queue=libnet>.
 
 =head1 SEE ALSO
 
-L<Net::Netrc>,
-L<Net::Cmd>
+L<Net::Cmd>.
+
+=head1 AUTHOR
+
+Graham Barr E<lt>L<gbarr@pobox.com|mailto:gbarr@pobox.com>E<gt>.
+
+Steve Hay E<lt>L<shay@cpan.org|mailto:shay@cpan.org>E<gt> is now maintaining
+libnet as of version 1.22_02.
 
 =head1 COPYRIGHT
 
-Versions up to 2.13 Copyright (c) 1995-1998 Graham Barr. All rights reserved.
-Changes in Version 2.13_01 onwards Copyright (C) 2013-2014 Steve Hay.  All
-rights reserved.
+Copyright (C) 1995-1998 Graham Barr.  All rights reserved.
 
-This program is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself.
+Copyright (C) 2013-2014, 2020 Steve Hay.  All rights reserved.
+
+=head1 LICENCE
+
+This module is free software; you can redistribute it and/or modify it under the
+same terms as Perl itself, i.e. under the terms of either the GNU General Public
+License or the Artistic License, as specified in the F<LICENCE> file.
+
+=head1 VERSION
+
+Version 3.14
+
+=head1 DATE
+
+22 May 2022
+
+=head1 HISTORY
+
+See the F<Changes> file.
 
 =cut
