@@ -1,9 +1,7 @@
 # Net::POP3.pm
 #
-# Versions up to 2.29 Copyright (c) 1995-2004 Graham Barr <gbarr@pobox.com>.
-# All rights reserved.
-# Changes in Version 2.29_01 onwards Copyright (C) 2013-2014 Steve Hay.  All
-# rights reserved.
+# Copyright (C) 1995-2004 Graham Barr.  All rights reserved.
+# Copyright (C) 2013-2016, 2020 Steve Hay.  All rights reserved.
 # This module is free software; you can redistribute it and/or modify it under
 # the same terms as Perl itself, i.e. under the terms of either the GNU General
 # Public License or the Artistic License, as specified in the F<LICENCE> file.
@@ -20,7 +18,7 @@ use IO::Socket;
 use Net::Cmd;
 use Net::Config;
 
-our $VERSION = "3.06";
+our $VERSION = "3.14";
 
 # Code for detecting if we can use SSL
 my $ssl_class = eval {
@@ -34,16 +32,19 @@ my $nossl_warn = !$ssl_class &&
   'To use SSL please install IO::Socket::SSL with version>=2.007';
 
 # Code for detecting if we can use IPv6
+my $family_key = 'Domain';
 my $inet6_class = eval {
   require IO::Socket::IP;
   no warnings 'numeric';
-  IO::Socket::IP->VERSION(0.20);
+  IO::Socket::IP->VERSION(0.25) || die;
+  $family_key = 'Family';
 } && 'IO::Socket::IP' || eval {
   require IO::Socket::INET6;
   no warnings 'numeric';
   IO::Socket::INET6->VERSION(2.62);
 } && 'IO::Socket::INET6';
 
+
 sub can_ssl   { $ssl_class };
 sub can_inet6 { $inet6_class };
 
@@ -63,7 +64,6 @@ sub new {
   }
   my $hosts = defined $host ? [$host] : $NetConfig{pop3_hosts};
   my $obj;
-  my @localport = exists $arg{ResvPort} ? (LocalPort => $arg{ResvPort}) : ();
 
   if ($arg{SSL}) {
     # SSL from start
@@ -78,7 +78,9 @@ sub new {
       PeerAddr => ($host = $h),
       PeerPort => $arg{Port} || 'pop3(110)',
       Proto => 'tcp',
-      @localport,
+      $family_key => $arg{Domain} || $arg{Family},
+      LocalAddr => $arg{LocalAddr},
+      LocalPort => exists($arg{ResvPort}) ? $arg{ResvPort} : $arg{LocalPort},
       Timeout => $arg{Timeout},
       )
       and last;
@@ -122,7 +124,7 @@ sub debug_text { $_[2] =~ /^(pass|rpop)/i ? "$1 ....\n" : $_[2]; }
 
 
 sub login {
-  @_ >= 1 && @_ <= 3 or croak 'usage: $pop3->login( USER, PASS )';
+  @_ >= 1 && @_ <= 3 or croak 'usage: $pop3->login([$user[, $pass]])';
   my ($me, $user, $pass) = @_;
 
   if (@_ <= 2) {
@@ -145,7 +147,7 @@ sub starttls {
 }
 
 sub apop {
-  @_ >= 1 && @_ <= 3 or croak 'usage: $pop3->apop( USER, PASS )';
+  @_ >= 1 && @_ <= 3 or croak 'usage: $pop3->apop([$user[, $pass]])';
   my ($me, $user, $pass) = @_;
   my $banner;
   my $md;
@@ -178,13 +180,13 @@ sub apop {
 
 
 sub user {
-  @_ == 2 or croak 'usage: $pop3->user( USER )';
+  @_ == 2 or croak 'usage: $pop3->user($user)';
   $_[0]->_USER($_[1]) ? 1 : undef;
 }
 
 
 sub pass {
-  @_ == 2 or croak 'usage: $pop3->pass( PASS )';
+  @_ == 2 or croak 'usage: $pop3->pass($pass)';
 
   my ($me, $pass) = @_;
 
@@ -223,7 +225,7 @@ sub last {
 
 
 sub top {
-  @_ == 2 || @_ == 3 or croak 'usage: $pop3->top( MSGNUM [, NUMLINES ])';
+  @_ == 2 || @_ == 3 or croak 'usage: $pop3->top($msgnum[, $numlines])';
   my $me = shift;
 
   return
@@ -245,7 +247,7 @@ sub popstat {
 
 
 sub list {
-  @_ == 1 || @_ == 2 or croak 'usage: $pop3->list( [ MSGNUM ] )';
+  @_ == 1 || @_ == 2 or croak 'usage: $pop3->list([$msgnum])';
   my $me = shift;
 
   return
@@ -266,7 +268,7 @@ sub list {
 
 
 sub get {
-  @_ == 2 or @_ == 3 or croak 'usage: $pop3->get( MSGNUM [, FH ])';
+  @_ == 2 or @_ == 3 or croak 'usage: $pop3->get($msgnum[, $fh])';
   my $me = shift;
 
   return
@@ -277,7 +279,7 @@ sub get {
 
 
 sub getfh {
-  @_ == 2 or croak 'usage: $pop3->getfh( MSGNUM )';
+  @_ == 2 or croak 'usage: $pop3->getfh($msgnum)';
   my $me = shift;
 
   return unless $me->_RETR(shift);
@@ -286,7 +288,7 @@ sub getfh {
 
 
 sub delete {
-  @_ == 2 or croak 'usage: $pop3->delete( MSGNUM )';
+  @_ == 2 or croak 'usage: $pop3->delete($msgnum)';
   my $me = shift;
   return 0 unless $me->_DELE(@_);
   ${*$me}{'net_pop3_deleted'} = 1;
@@ -294,7 +296,7 @@ sub delete {
 
 
 sub uidl {
-  @_ == 1 || @_ == 2 or croak 'usage: $pop3->uidl( [ MSGNUM ] )';
+  @_ == 1 || @_ == 2 or croak 'usage: $pop3->uidl([$msgnum])';
   my $me = shift;
   my $uidl;
 
@@ -317,7 +319,7 @@ sub uidl {
 
 
 sub ping {
-  @_ == 2 or croak 'usage: $pop3->ping( USER )';
+  @_ == 2 or croak 'usage: $pop3->ping($user)';
   my $me = shift;
 
   return () unless $me->_PING(@_) && $me->message =~ /(\d+)\D+(\d+)/;
@@ -623,27 +625,30 @@ Net::POP3 - Post Office Protocol 3 Client class (RFC1939)
 This module implements a client interface to the POP3 protocol, enabling
 a perl5 application to talk to POP3 servers. This documentation assumes
 that you are familiar with the POP3 protocol described in RFC1939.
+With L<IO::Socket::SSL> installed it also provides support for implicit and
+explicit TLS encryption, i.e. POP3S or POP3+STARTTLS.
 
 A new Net::POP3 object must be created with the I<new> method. Once
 this has been done, all POP3 commands are accessed via method calls
 on the object.
 
-The Net::POP3 class is a subclass of Net::Cmd and IO::Socket::INET.
+The Net::POP3 class is a subclass of Net::Cmd and (depending on avaibility) of
+IO::Socket::IP, IO::Socket::INET6 or IO::Socket::INET.
 
-=head1 CONSTRUCTOR
+=head2 Class Methods
 
 =over 4
 
-=item new ( [ HOST ] [, OPTIONS ] )
+=item C<new([$host][, %options])>
 
-This is the constructor for a new Net::POP3 object. C<HOST> is the
+This is the constructor for a new Net::POP3 object. C<$host> is the
 name of the remote host to which an POP3 connection is required.
 
-C<HOST> is optional. If C<HOST> is not given then it may instead be
+C<$host> is optional. If C<$host> is not given then it may instead be
 passed as the C<Host> option described below. If neither is given then
 the C<POP3_Hosts> specified in C<Net::Config> will be used.
 
-C<OPTIONS> are passed in a hash like fashion, using key and value pairs.
+C<%options> are passed in a hash like fashion, using key and value pairs.
 Possible options are:
 
 B<Host> - POP3 host to connect to. It may be a single scalar, as defined for
@@ -659,9 +664,14 @@ upgrade with C<starttls>.
 You can use SSL arguments as documented in L<IO::Socket::SSL>, but it will
 usually use the right arguments already.
 
-B<ResvPort> - If given then the socket for the C<Net::POP3> object
-will be bound to the local port given using C<bind> when the socket is
-created.
+B<LocalAddr> and B<LocalPort> - These parameters are passed directly
+to IO::Socket to allow binding the socket to a specific local address and port.
+For compatibility with older versions B<ResvPort> can be used instead of
+B<LocalPort>.
+
+B<Domain> - This parameter is passed directly to IO::Socket and makes it
+possible to enforce IPv4 connections even if L<IO::Socket::IP> is used as super
+class. Alternatively B<Family> can be used.
 
 B<Timeout> - Maximum time, in seconds, to wait for a response from the
 POP3 server (default: 120)
@@ -670,7 +680,7 @@ B<Debug> - Enable debugging information
 
 =back
 
-=head1 METHODS
+=head2 Object Methods
 
 Unless otherwise stated all methods return either a I<true> or I<false>
 value, with I<true> meaning that the operation was a success. When a method
@@ -683,26 +693,26 @@ documented here.
 
 =over 4
 
-=item host ()
+=item C<host()>
 
 Returns the value used by the constructor, and passed to IO::Socket::INET,
 to connect to the host.
 
-=item auth ( USERNAME, PASSWORD )
+=item C<auth($username, $password)>
 
 Attempt SASL authentication.
 
-=item user ( USER )
+=item C<user($user)>
 
 Send the USER command.
 
-=item pass ( PASS )
+=item C<pass($pass)>
 
 Send the PASS command. Returns the number of messages in the mailbox.
 
-=item login ( [ USER [, PASS ]] )
+=item C<login([$user[, $pass]])>
 
-Send both the USER and PASS commands. If C<PASS> is not given the
+Send both the USER and PASS commands. If C<$pass> is not given the
 C<Net::POP3> uses C<Net::Netrc> to lookup the password using the host
 and username. If the username is not specified then the current user name
 will be used.
@@ -713,25 +723,25 @@ will give a true value in a boolean context, but zero in a numeric context.
 
 If there was an error authenticating the user then I<undef> will be returned.
 
-=item starttls ( SSLARGS )
+=item C<starttls(%sslargs)>
 
 Upgrade existing plain connection to SSL.
 You can use SSL arguments as documented in L<IO::Socket::SSL>, but it will
 usually use the right arguments already.
 
-=item apop ( [ USER [, PASS ]] )
+=item C<apop([$user[, $pass]])>
 
-Authenticate with the server identifying as C<USER> with password C<PASS>.
+Authenticate with the server identifying as C<$user> with password C<$pass>.
 Similar to L</login>, but the password is not sent in clear text.
 
 To use this method you must have the Digest::MD5 or the MD5 module installed,
 otherwise this method will return I<undef>.
 
-=item banner ()
+=item C<banner()>
 
 Return the sever's connection banner
 
-=item capa ()
+=item C<capa()>
 
 Return a reference to a hash of the capabilities of the server.  APOP
 is added as a pseudo capability.  Note that I've been unable to
@@ -739,112 +749,134 @@ find a list of the standard capability values, and some appear to
 be multi-word and some are not.  We make an attempt at intelligently
 parsing them, but it may not be correct.
 
-=item  capabilities ()
+=item C<capabilities()>
 
 Just like capa, but only uses a cache from the last time we asked
 the server, so as to avoid asking more than once.
 
-=item top ( MSGNUM [, NUMLINES ] )
+=item C<top($msgnum[, $numlines])>
 
-Get the header and the first C<NUMLINES> of the body for the message
-C<MSGNUM>. Returns a reference to an array which contains the lines of text
+Get the header and the first C<$numlines> of the body for the message
+C<$msgnum>. Returns a reference to an array which contains the lines of text
 read from the server.
 
-=item list ( [ MSGNUM ] )
+=item C<list([$msgnum])>
 
 If called with an argument the C<list> returns the size of the message
 in octets.
 
 If called without arguments a reference to a hash is returned. The
-keys will be the C<MSGNUM>'s of all undeleted messages and the values will
+keys will be the C<$msgnum>'s of all undeleted messages and the values will
 be their size in octets.
 
-=item get ( MSGNUM [, FH ] )
+=item C<get($msgnum[, $fh])>
 
-Get the message C<MSGNUM> from the remote mailbox. If C<FH> is not given
+Get the message C<$msgnum> from the remote mailbox. If C<$fh> is not given
 then get returns a reference to an array which contains the lines of
-text read from the server. If C<FH> is given then the lines returned
-from the server are printed to the filehandle C<FH>.
+text read from the server. If C<$fh> is given then the lines returned
+from the server are printed to the filehandle C<$fh>.
 
-=item getfh ( MSGNUM )
+=item C<getfh($msgnum)>
 
 As per get(), but returns a tied filehandle.  Reading from this
 filehandle returns the requested message.  The filehandle will return
 EOF at the end of the message and should not be reused.
 
-=item last ()
+=item C<last()>
 
-Returns the highest C<MSGNUM> of all the messages accessed.
+Returns the highest C<$msgnum> of all the messages accessed.
 
-=item popstat ()
+=item C<popstat()>
 
 Returns a list of two elements. These are the number of undeleted
 elements and the size of the mbox in octets.
 
-=item ping ( USER )
+=item C<ping($user)>
 
 Returns a list of two elements. These are the number of new messages
-and the total number of messages for C<USER>.
+and the total number of messages for C<$user>.
 
-=item uidl ( [ MSGNUM ] )
+=item C<uidl([$msgnum])>
 
-Returns a unique identifier for C<MSGNUM> if given. If C<MSGNUM> is not
+Returns a unique identifier for C<$msgnum> if given. If C<$msgnum> is not
 given C<uidl> returns a reference to a hash where the keys are the
 message numbers and the values are the unique identifiers.
 
-=item delete ( MSGNUM )
+=item C<delete($msgnum)>
 
-Mark message C<MSGNUM> to be deleted from the remote mailbox. All messages
+Mark message C<$msgnum> to be deleted from the remote mailbox. All messages
 that are marked to be deleted will be removed from the remote mailbox
 when the server connection closed.
 
-=item reset ()
+=item C<reset()>
 
 Reset the status of the remote POP3 server. This includes resetting the
 status of all messages to not be deleted.
 
-=item quit ()
+=item C<quit()>
 
 Quit and close the connection to the remote POP3 server. Any messages marked
 as deleted will be deleted from the remote mailbox.
 
-=item can_inet6 ()
+=item C<can_inet6()>
 
 Returns whether we can use IPv6.
 
-=item can_ssl ()
+=item C<can_ssl()>
 
 Returns whether we can use SSL.
 
 =back
 
-=head1 NOTES
+=head2 Notes
 
 If a C<Net::POP3> object goes out of scope before C<quit> method is called
 then the C<reset> method will called before the connection is closed. This
 means that any messages marked to be deleted will not be.
 
+=head1 EXPORTS
+
+I<None>.
+
+=head1 KNOWN BUGS
+
+See L<https://rt.cpan.org/Dist/Display.html?Status=Active&Queue=libnet>.
+
 =head1 SEE ALSO
 
 L<Net::Netrc>,
 L<Net::Cmd>,
-L<IO::Socket::SSL>
+L<IO::Socket::SSL>.
 
 =head1 AUTHOR
 
-Graham Barr E<lt>F<gbarr@pobox.com>E<gt>
+Graham Barr E<lt>L<gbarr@pobox.com|mailto:gbarr@pobox.com>E<gt>.
 
-Steve Hay E<lt>F<shay@cpan.org>E<gt> is now maintaining libnet as of version
-1.22_02
+Steve Hay E<lt>L<shay@cpan.org|mailto:shay@cpan.org>E<gt> is now maintaining
+libnet as of version 1.22_02.
 
 =head1 COPYRIGHT
 
-Versions up to 2.29 Copyright (c) 1995-2004 Graham Barr. All rights reserved.
-Changes in Version 2.29_01 onwards Copyright (C) 2013-2014 Steve Hay.  All
-rights reserved.
+Copyright (C) 1995-2004 Graham Barr.  All rights reserved.
+
+Copyright (C) 2013-2016, 2020 Steve Hay.  All rights reserved.
+
+=head1 LICENCE
 
 This module is free software; you can redistribute it and/or modify it under the
 same terms as Perl itself, i.e. under the terms of either the GNU General Public
 License or the Artistic License, as specified in the F<LICENCE> file.
 
+=head1 VERSION
+
+Version 3.14
+
+=head1 DATE
+
+22 May 2022
+
+=head1 HISTORY
+
+See the F<Changes> file.
+
 =cut
